<?php
/*
Plugin Name: Wordpress Security by Robert Huie
Plugin URI: http://wordpress.digitalpixies.com/dpt-security
Description: Security Improvements to standard wordpress installs without being too annoying to end users.
Version: 2.0.0
Author: Robert Huie
Author URI: http://DigitalPixies.com
License: GPLv2
*/

if(!class_exists("dpt_security")) {
	class dpt_security {
		public static $data = null;
		public static function RegisterHooks() {
      wp_enqueue_script('jquery');
			add_action('rest_api_init', __CLASS__.'::RESTAPIEndpoints');
			add_action('login_head', __CLASS__.'::ReformatFormJS');
			add_filter('wp_authenticate_user', __CLASS__.'::AuthenticateUser');
			// we want to run after this one add_filter('authenticate', 'wp_authenticate_username_password', 20, 3);
			add_filter('authenticate', __CLASS__.'::RotateModulation', 30, 3);
			add_filter('login_form_bottom', __CLASS__.'::RenderRecaptcha');

# Fires immediately after an existing user is updated.
#do_action( 'profile_update', $user_id, $old_user_data );

# Fires immediately after a new user is registered.
#do_action( 'user_register', $user_id );

//			add_filter('registration_errors', 'dpt_security::ValidateUserRegistration', 10, 3);
			add_filter('user_profile_update_errors', __CLASS__.'::ValidatePasswordUpdate', 10, 3);
//			add_action('check_passwords', 'dpt_security::ValidatePassword');
			if(is_admin()) {
				add_action('admin_menu', __CLASS__.'::AdminMenu');
				add_action('admin_enqueue_scripts', __CLASS__.'::EnableAdminCSSJS');
			}
			add_action('login_enqueue_scripts', __CLASS__.'::EnableCSSJS');
		}
		public static function RenderRecaptcha() {
			return 'recaptcha here';
		}
		public static function RESTAPIEndpoints() {

			remove_filter( 'rest_pre_serve_request', 'rest_send_cors_headers' );
			add_filter( 'rest_pre_serve_request', function( $value ) {
				header( 'Access-Control-Allow-Origin: *' );
				header( 'Access-Control-Allow-Methods: GET' );
				header( 'Access-Control-Allow-Credentials: true' );
				header( 'Access-Control-Expose-Headers: Link', false );
				header( 'Access-Control-Allow-Headers: X-WP-Nonce');

		    if ( 'OPTIONS' == $_SERVER['REQUEST_METHOD'] ) {
		        status_header(200);
		        exit();
		    }
				return $value;
			} );

			register_rest_route('dpt-security/v1', '/settings', array(
				'methods' => 'GET',
				'callback' => __CLASS__.'::ListSettings',
				'permission_callback' => __CLASS__.'::PermissionToREST'
			));
			register_rest_route('dpt-security/v1', '/settings', array(
				'methods' => 'POST',
				'callback' => __CLASS__.'::UpdateSettings',
				'permission_callback' => __CLASS__.'::PermissionToREST'
			));
		}
		public static function PermissionToREST($request) {
			if(current_user_can('administrator')) {
				return true;
			}
			return false;
		}
		public static function ListSettings(WP_REST_Request $request) {
			global $wpdb;
			$output = array();
			$output["greylist"]=get_option(__CLASS__."greylist")==1;
			$output["trigger"]=get_option(__CLASS__."trigger");
			$output["use_identifyspam_greylist"]=get_option(__CLASS__."use_identifyspam_greylist")==1;
			$output["recaptcha"]["secret_key"]=get_option(__CLASS__."recaptchasecret_key");
			$output["recaptcha"]["site_key"]=get_option(__CLASS__."recaptchasite_key");
			$output["data_dir"]=wp_upload_dir();
			$output["data_dir"]=$output["data_dir"]["basedir"]."/dpt-security";
			return $output;
		}
		private static function VerifyRecaptcha($secret, $response) {
			$data=array(
				'secret'=>$secret,
				'response'=>$response
				);
			$postData=http_build_query($data);
			$options=array(
				"ssl"=>array(
					"verify_peer"=>false,
					"verify_peer_name"=>false),
				'http'=>array(
					'method'=>'POST',
					'header'=>"Content-type: application/x-www-form-urlencoded\r\n"
						.'Content-Length: '.strlen($postData),//."\r\n"
					'content'=>$postData
				));
			$context=stream_context_create($options);
			$output=json_decode(file_get_contents("https://www.google.com/recaptcha/api/siteverify", false, $context),$assoc=true);
			return $output["success"]==true;
		}
		public static function DisplayRecaptcha() {
			$trigger = get_option(__CLASS__."trigger");
			switch($trigger) {
				case "greylist_logins":
					include_once("includes/php/KarmaSearch.Class.php");
					$dir=wp_upload_dir();
					$dir=$dir["basedir"]."/dpt-security/karmasearch";
					$karmaSearch = new KarmaSearch($dir);
					return $karmaSearch->CheckIP($_SERVER['REMOTE_ADDR']);
				case "all_logins":
					return true;
				case "disabled":
					return false;
			}
		}
		public static function UpdateSettings(WP_REST_Request $request) {
			global $wpdb;
			$param=$request->get_json_params();
			if(isset($param["verify"]) && $param["verify"]) {
				$output = array();
				$output["success"]=dpt_security::VerifyRecaptcha($param["recaptcha"]["secret_key"], $param["recaptcha"]["response"]);
				$output["message"]="Recaptcha test was successful";
				return $output;
			}
			foreach($param as $key=>$value) {
				switch($key) {
					case "greylist_dir":
						break;
					case "greylist":
					case "trigger":
					case "use_identifyspam_greylist":
						update_option( __CLASS__.$key, $value, $autoload=false);
						break;
					case "recaptcha":
						update_option( __CLASS__.$key."secret_key", $value["secret_key"], $autoload=false);
						update_option( __CLASS__.$key."site_key", $value["site_key"], $autoload=false);
						break;
				}
			}
			//update_option( __CLASS__.$param, $new_value, $autoload=false );
			$output["success"]=true;
			$output["message"]="Saved successfully";
			return $output;
		}
		public static function EnableAdminCSSJS($hook) {
			wp_register_style('localizedbootstrap', plugin_dir_url(__FILE__).'includes/styles/localizedbootstrap.css');
      wp_register_style('localizedbootstrap', plugins_url('includes/styles/localizedbootstrap-theme.css', __FILE__));
      wp_enqueue_style('localizedbootstrap');

			wp_register_style(__CLASS__, plugin_dir_url(__FILE__).'includes/styles/styles.css');
      wp_enqueue_style(__CLASS__);

			wp_register_script('angular-ui', plugin_dir_url(__FILE__).'includes/scripts/vendor.js', array(), "2.5.0", true);
      wp_enqueue_script('angular-ui');

			wp_register_script(__CLASS__, plugin_dir_url(__FILE__).'includes/scripts/scripts.js', array("angular-ui"), "2.5.0", true);
			wp_enqueue_script(__CLASS__);

			$params['ajax_url']=admin_url('admin-ajax.php');
      $params['rest_url']=esc_url_raw(rest_url());
			$params['nonce']=wp_create_nonce('wp_rest');
			wp_localize_script(__CLASS__, __CLASS__, $params);
		}
		public static function EnableCSSJS($hook) {
			wp_register_style(__CLASS__, plugin_dir_url(__FILE__).'includes/styles/styles.css');
      wp_enqueue_style(__CLASS__);
    }
		public static function ValidatePasswordUpdate($errors, $update, $user) {
//			print "<pre>".htmlentities(print_r($user, true))."</pre>";
			if(!preg_match('/[a-z]/', $user->user_pass)) {
				$errors->add('pass', __('<strong>ERROR</strong>: Password must have at least one lower case character.'), array('form-field' => 'pass1-text'));
			}
			if(!preg_match('/[A-Z]/', $user->user_pass)) {
				$errors->add('pass', __('<strong>ERROR</strong>: Password must have at least one upper case character.'), array('form-field' => 'pass1-text'));
			}
			if(!preg_match('/[0-9]/', $user->user_pass)) {
				$errors->add('pass', __('<strong>ERROR</strong>: Password must have at least one number.'), array('form-field' => 'pass1-text'));
			}
			if(!preg_match('/[^\da-z]/i', $user->user_pass)) {
				$errors->add('pass', __('<strong>ERROR</strong>: Password must have at least one non alphanumeric.'), array('form-field' => 'pass1-text'));
			}
			if(strlen($user->user_pass)<6) {
				$errors->add('pass', __('<strong>ERROR</strong>: Password must be at least 6 characters .'), array('form-field' => 'pass1-text'));
			}
			//should prevent allowing repeating characters
		}
		public static function ValidatePassword($user_login, $pass1, $pass2) {

		}
/*
		public static function ValidateUserRegistration($errors, $sanitized_user_login, $user_email) {
			$errors->add('password_error', __('<strong>ERROR</strong>: Invalid password',print_r($_REQUEST, true)));
	    return $errors;
		}
*/
		public static function RotateModulation($user, $username, $password) {
			dpt_security::$data["modulation_value"]=uniqid();
			include_once("includes/php/KarmaSearch.Class.php");
			$dir=wp_upload_dir();
			$dir=$dir["basedir"]."/dpt-security/karmasearch";
			$karmaSearch = new KarmaSearch($dir);
			if (is_a($user, 'WP_User')) {
				$karmaSearch->RemoveIP($_SERVER['REMOTE_ADDR']);
				return $user;
			}
			if (is_wp_error($user) && 'incorrect_password' == $user->get_error_code()) {
//				$user->add('debug', "found incorrect password");
				$karmaSearch->LogIP($_SERVER['REMOTE_ADDR']);
			}
			return $user;
		}
		public static function Initialize() {
			session_start();
			if(!isset($_SESSION[__CLASS__]))
				$_SESSION[__CLASS__]=array(
					"modulator_name"=>md5(uniqid())
					, "modulation_value"=>uniqid());
			dpt_security::$data=&$_SESSION[__CLASS__];
		}
		public function __construct() {
			dpt_security::Initialize();
			register_activation_hook(__FILE__,__CLASS__.'::Activate');
			add_action('init', __CLASS__.'::RegisterHooks');
		}
		public static function ReformatFormJS() {
			dpt_security::Initialize();
			$data = &dpt_security::$data;
			if(dpt_security::DisplayRecaptcha()) {
				$site_key = get_option(__CLASS__."recaptchasite_key");
				print <<<EOF
<script type="text/javascript">
	function onloadRenderRecaptcha() {
		(function($) {
			$('<p></p>').append('<div id="recaptcha-contents"></div>').insertBefore('#loginform .submit');
			grecaptcha.render('recaptcha-contents', {
				'sitekey' : '${site_key}'
			});
		})(jQuery);
	}
</script>
<script src="https://www.google.com/recaptcha/api.js?onload=onloadRenderRecaptcha&render=explicit"
		async defer>
</script>
EOF;
			}
			print <<<EOF
<script type="text/javascript">
	(function($) {
		$(document).ready(function() {
			$("#loginform").append('<input type="hidden" name="{$data["modulator_name"]}" value="{$data["modulation_value"]}" />');
		});
	})(jQuery);
</script>
EOF;
		}
		public static function AuthenticateUser($username, $password="") {
			dpt_security::Initialize();
			if($_REQUEST[dpt_security::$data["modulator_name"]]!=dpt_security::$data["modulation_value"]) {
				$error = new WP_Error();
				$error->add('no_javascript', "Javascript needs to be enabled.");
				return $error;
			}
			if(isset($_REQUEST["g-recaptcha-response"])) {
				$secret_key = get_option(__CLASS__."recaptchasecret_key");
				if(dpt_security::VerifyRecaptcha($secret_key, $_REQUEST["g-recaptcha-response"])) {
					return $username;
				}
				$error = new WP_Error();
				$error->add('recaptcha_failed', "Captcha failed.");
				return $error;
			}
			return $username;
		}
		public static function AdminMenu() {
			add_options_page('Wordpress Security by Robert Huie', 'Security', 'manage_options', __CLASS__, 'dpt_security::AdminHTML');
		}
		public static function AdminHTML() {
			print <<<EOF
<div class="wrap dpt">
	<header>
		<h1>Wordpress Security</h1>
		<p class="tagline">by Robert Huie</p>
	</header>
	<div ng-app="yeomanApp" ng-controller="SettingsCtrl" class="localizedbootstrap container-fluid">
EOF;
			include_once(dirname(__FILE__).'/includes/views/settings.html');
			print <<<EOF
	</div>
</div>
<script src="https://www.google.com/recaptcha/api.js?render=explicit"
		async defer>
</script>
EOF;
		}
		public static function Activate() {
		    $upload = wp_upload_dir();
		    $upload_dir = $upload['basedir'];
		    $upload_dir = $upload_dir.'/dpt-security';
				$karmasearch_dir = $upload_dir."/karmasearch";
		    if(!is_dir($upload_dir)) {
		       mkdir($upload_dir, 0700);
		    }
				if(!is_dir($karmasearch_dir)) {
					mkdir($karmasearch_dir, 0700);
			 }
		}
	}
}

$dpt_security = new dpt_security();
