'use strict';

/**
 * @ngdoc overview
 * @name yeomanApp
 * @description
 * # yeomanApp
 *
 * Main module of the application.
 */
angular
  .module('yeomanApp', [
    'ngAnimate',
    'ngCookies',
    'ngResource',
    'ngRoute',
    'ngSanitize',
    'ui.bootstrap',
    'ngTouch'
  ])
  .constant('AppConfig', {
    SettingsURL: dpt_security.rest_url+'dpt-security/v1/:id'
  })
  .config(function ($routeProvider, $locationProvider) {
    $locationProvider.hashPrefix('');
    $routeProvider
      .when('/', {
        templateUrl: 'views/main.html',
        controller: 'MainCtrl',
        controllerAs: 'main'
      })
      .when('/settings', {
        templateUrl: 'views/settings.html',
        controller: 'SettingsCtrl',
        controllerAs: 'settings'
      })
      .otherwise({
        redirectTo: '/'
      });
  });

'use strict';

/**
 * @ngdoc function
 * @name yeomanApp.controller:MainCtrl
 * @description
 * # MainCtrl
 * Controller of the yeomanApp
 */
angular.module('yeomanApp')
  .controller('MainCtrl', function () {
  });

'use strict';

/**
 * @ngdoc function
 * @name yeomanApp.controller:SettingsCtrl
 * @description
 * # SettingsCtrl
 * Controller of the yeomanApp
 */
angular.module('yeomanApp')
  .controller('SettingsCtrl', function ($scope, Settings, $window, $uibModal, $document, $http) {
    function ClearMessages() {
      while(messages.length)
        messages.pop();
    }
    function TestedRecaptcha() {
      switch(data.trigger) {
        case "disabled":
          return true;
        default:
          return testedRecaptcha;
      }
    }

    var messages = [];
    $scope.messages=messages;
    var testedRecaptcha = false;
    var data = {};
    $scope.data = data;
    data.use_identifyspam_greylist = false;
    data.greylist = false;
    data.trigger = "disabled";
    var recaptcha = {};
    $scope.recaptcha = recaptcha;
    recaptcha.data = {
      'site_key':'',
      'secret_key':''
    };
    recaptcha.modalInstance=null;
    recaptcha.ShowRecaptchaTest = function() {
      recaptcha.modalInstance = $uibModal.open({
        animation: true,
        templateUrl: 'recaptcha-modal.html',
        size: 'lg',
        appendTo: angular.element('#modal-placeholder'),
        controller: function($scope) {
          $scope.recaptcha = recaptcha; //buttons in dialog targets this
        }
      });
      recaptcha.modalInstance.rendered.then(function() {
//        console.log(["modal:opened", angular.element('#recaptcha-contents'), recaptcha]);
        recaptcha.widget_id = grecaptcha.render('recaptcha-contents', {
          'sitekey' : recaptcha.data.site_key
        });
      });
    };
    recaptcha.CancelTest = function() {
      recaptcha.modalInstance.close();
    };
    recaptcha.SubmitTest = function() {
      settings = new Settings({id:"settings"});
      angular.extend(settings, data, {'recaptcha':recaptcha.data,'verify':true});
      settings.recaptcha.response=grecaptcha.getResponse(recaptcha.widget_id);
      ClearMessages();
      settings.$save(function(response) {
//        console.log(["results of verify",response]);
        if(response.success)
          testedRecaptcha = true;
        response.type = response.success?'success':'danger';
        response.text = response.message;
        messages.push(response);
      });
      recaptcha.modalInstance.close();
    };

    $scope.Save = function() {
      ClearMessages();
      if(!TestedRecaptcha()) {
        messages.push({'type':'danger','text':'Recaptcha keys must be tested first to confirm they work'})
        return;
      }
      var payload = angular.copy(data);
      payload.recaptcha = angular.copy(recaptcha.data);
//      var settings = new Settings(payload);
      settings = new Settings({id:"settings"});
      angular.extend(settings, data, {'recaptcha':recaptcha.data});
      settings.$save({id:"settings"}, function(response) {
        response.type = response.success?'success':'danger';
        response.text = response.message;
        messages.push(response);
      });
    };

    if(typeof $window.dpt_security === 'undefined')
      $window.dpt_security = {nonce:''};
    $scope.nonce = $window.dpt_security.nonce;

    var settings = Settings.get({id:"settings"}, function() {
      data.greylist = angular.copy(settings.greylist);
      data.use_identifyspam_greylist = angular.copy(settings.use_identifyspam_greylist);
      data.trigger = angular.copy(settings.trigger);
      data.data_dir = angular.copy(settings.data_dir);
      recaptcha.data.secret_key = angular.copy(settings.recaptcha.secret_key);
      recaptcha.data.site_key = angular.copy(settings.recaptcha.site_key);
    });

  });

'use strict';

/**
 * @ngdoc service
 * @name yeomanApp.Settings
 * @description
 * # Settings
 * Factory in the yeomanApp.
 */
angular.module('yeomanApp')
  .factory('Settings', function ($resource, AppConfig, $window) {
    if(typeof $window.dpt_security === 'undefined')
      $window.dpt_security = {nonce:'ba4e5617f2'};
    dpt_security = $window.dpt_security;
    var api = $resource(AppConfig.SettingsURL
      , {id:'@id'}
      , {
        query:{
          headers: {'X-WP-Nonce': dpt_security.nonce},
          isArray:true
        },
        get:{
          headers: {'X-WP-Nonce': dpt_security.nonce}
        },
        save:{
          method:'POST',
          headers: {'X-WP-Nonce': dpt_security.nonce}
        }
      }
    );
    return api;
  });
